<?php

function diff_time( $st = FALSE )
{
	static $ts = FALSE;
	if ( $ts === FALSE ) {
		$ts = $st ? $st : microtime();
		return TRUE;
	}
	list( $ts_usec, $ts_sec ) = explode( ' ', $ts );
	list( $te_usec, $te_sec ) = explode( ' ', microtime() );
	return ( (float)$te_usec - (float)$ts_usec ) + (float)( (int)$te_sec - (int)$ts_sec );
}

function escape( $str )
{
	return strtr( (string)$str, "\n\r\t\0", '    ' );
}

function get_hostname()
{
	$host = $_SERVER['HTTP_HOST'];
	$port = (int)$_SERVER['SERVER_PORT'];

	if ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === 'on' ) {
		$scheme = 'https://';
		$port   = ( $port !== 443 ) ? ':' . $port : '';
	}
	else {
		$scheme = 'http://';
		$port   = ( $port !== 80  ) ? ':' . $port : '';
	}
	return $scheme . $host . $port;
}

function h( $str )
{
	return htmlspecialchars( (string)$str, ENT_QUOTES, 'UTF-8' );
}

function ifsetor( &$var, $default = NULL )
{
	if ( ! isset( $var ) ) {
		return $default;
	}
	return $var;
}

/*
 * see save_config
 */
function load_config( $file )
{
	if ( ! is_readable( $file ) || @filesize( $file ) === 0 ) {
		return array();
	}
	return @include( $file );
}

/**
 * rm -rf [dir]
 */
function remove_dir( $dir )
{
	$dir = rtrim( $dir, '/' );
	if ( is_dir( $dir ) ) {
		$dh = opendir( $dir );
		while ( FALSE !== ( $file = readdir( $dh ) ) ) {
			if ( $file === '.' || $file === '..' ) {
				continue;
			}
			remove_dir( $dir . '/' . $file );
		}
		rmdir( $dir );
		return;
	}
	unlink( $dir );
}

/*
 * see load_config
 */
function save_config( $file, $array )
{
	if ( ! is_writable( $file ) && ! @touch( $file ) ) {
		return FALSE;
	}
	$tmpfile = $file . '.' . uniqid( rand() );
	$result = save_array( $tmpfile, $array );
	if ( ! $result ) {
		return FALSE;
	}
	// Win32 cannot support direct rename();
	if ( substr( PHP_OS, 0, 3 ) === 'WIN' ) {
		@unlink( $file );
	}
	clearstatcache();

	return rename( $tmpfile, $file );
}

function unify_line_feed( $str )
{
	return strtr( str_replace( "\x0D\x0A", "\n", (string)$str ), "\x0D\x0A", "\n\n" );
}

if ( version_compare( PHP_VERSION, '5.0.0', '>=' ) ) {

	function save_array( $filename, $array )
	{
		$result = file_put_contents( $filename, '<?php return ' . var_export( $array, TRUE ) . ';' );
		return is_int( $result );
	}
}
else {
	// for var_export() in PHP 4.3.10/4.3.10RC1/4.3.10RC2 bug. see http://bugs.php.net/bug.php?id=31072
	if ( version_compare( PHP_VERSION, '4.3.10', '<=' ) && version_compare( PHP_VERSION, '4.3.9',  '>' ) ) {

		function save_array( $filename, $array )
		{
			if ( ! empty( $array[''] ) ) {
				$code = var_export( array( $md5 = md5( microtime() ) => $array[''] ) + $array, TRUE );
				$code = preg_replace( "/" . $md5 . "/", '', $code );
			}
			else {
				$code = var_export( $array, TRUE );
			}
			$result = file_put_contents( $filename, '<?php return ' . $code . ';' );
			return is_int( $result );
		}
	}
	else {

		function save_array( $filename, $array )
		{
			// for var_export() bug. see http://bugs.php.net/bug.php?id=25758
			if ( version_compare( PHP_VERSION, '4.3.4', '<' ) ) {
				$arr = array();
				foreach ( $array as $key => $val ) {
					$key = str_replace( array( '\\', '\'' ), array( '\\\\', '\\\'' ), $key );
					$arr[$key] = $val;
				}
				$array = $arr;
			}
			$result = file_put_contents( $filename, '<?php return ' . var_export( $array, TRUE ) . ';' );
			return is_int( $result );
		}
	}

	function file_put_contents( $filename, $contents )
	{
		$result = FALSE;
		$mode   = is_file( $filename ) ? 'r+' : 'w';

		$fp = fopen( $filename, $mode );
		if ( ! $fp ) {
			return $result;
		}
		flock( $fp, LOCK_EX );

		rewind( $fp );
		$num = fwrite( $fp, $contents );
		if ( is_int( $num ) ) {
			$result = ftruncate( $fp, ftell( $fp ) );
		}
		fflush( $fp );
		flock( $fp, LOCK_UN );
		fclose( $fp );

		return ( $result !== FALSE ) ? $num : FALSE;
	}
}
