<?php

class Command
{
	function addCategory( $category )
	{
		$cate =& Category::getInstance();
		$result = $cate->addCategory( $category );

		return array( $result ? 'INFO_ADD_CATEGORY' : 'ERROR_ADD_CATEGORY', $category );
	}

	function addItem( $uri )
	{
		if ( ! Validate::isUri( $uri ) ) {
			return array( 'ERROR_INVALID_URI', $uri, $err_code );
		}
		$code = Command::checkSite( $uri );
		if ( $code !== 'OK' ) {
			return array( 'ERROR_FAILD_ADD_ITEM', $uri, $code );
		}
		return array( 'INFO_ADD_ITEM', $uri, $code );
	}

	function alterCategory( $id, $name )
	{
		$cate =& Category::getInstance();
		$category = $cate->get( $id );
		$result   = $cate->alterCategory( $id, $name );

		return array( $result ? 'INFO_ALTER_CATEGORY' : 'ERROR_ALTER_CATEGORY', $category, $name );
	}

	function alterItem( $uri, $arr )
	{
		$result = Command::existItem( $uri );
		if ( ! $result ) {
			return array( 'ERROR_NOT_FILE', $uri );
		}
		if ( ! Validate::isUri( $arr['to_uri'] ) ) {
			return array( 'ERROR_INVALID_URI', $arr['to_uri'] );
		}
		$item =& new Item( $uri );
		$item->setProperties( $arr );
		$item->saveProperties();
		if ( $uri !== $arr['to_uri'] ) {
			$item->setProperties( array( 'uri' => $arr['to_uri'] ) );
			$item->saveProperties();
			$item->moveProperties( $arr['to_uri'] );
		}
		return array( 'INFO_CHANGE_SETTING', $arr['to_uri'] );
	}

	function checkSite( $uri )
	{
		require_once( LIBS_DIR . 'baselib/net.inc.php' );

		$item =& new Item( $uri );
		$result = $item->check();
		if ( $result || $item->isFile() ) {
			$result = $item->saveLog();
			if ( ! $result ) {
				trigger_error(
				   	htmlspecialchars( "Failed to write log file. URI : " . $uri, ENT_QUOTES, 'UTF-8' ),
					E_USER_ERROR
				);
			}
		}
		return $item->getError();
	}

	function changePass( $user, $pass1, $pass2 )
	{
		$conf =& Config::getInstance();
		if ( ! $user || ! $pass1 || ! $pass2 ) {
			return 'ERROR_NO_USER_PASS';
		}
		if ( $pass1 !== $pass2 ) {
			return 'ERROR_NOT_VERIFY_PASS';
		}
		$conf->set( 'user', $user );
		$conf->set( 'pass', md5( $pass1 ) );

		return 'INFO_CHANGE_USER_PASS';
	}

	function checkAll( $count, $uri, $count_file )
	{
		$code = Command::checkSite( $uri );

		// increment count
		$all = (int)file_get_contents( $count_file );
		touch( $count_file, $num = filemtime( $count_file ) + 1 );

		return sprintf( "%4d/%4d/%4d %-13s : %s\n", $num, $count, $all, $code, $uri );
	}

	function deleteItem( $uri )
	{
		$item =& new Item( $uri );
		$result = $item->delete();
		if ( ! $result ) {
			return array( 'ERROR_NOT_FILE', $uri );
		}
		return array( 'INFO_DEL_ITEM', $uri );
	}

	function delCategory( $id )
	{
		$cate =& Category::getInstance();
		$category = $cate->get( $id );
		$result   = $cate->delCategory( $id );

		// change items category from deleted category to no category.
		@include( SELF_DIR . 'misc/debug/reset_category.php' );
		reset_category( $id );

		return array( $result ? 'INFO_DEL_CATEGORY' : 'ERROR_DEL_CATEGORY', $category );
	}

	function diff( $old, $new, $tag = FALSE )
	{
		static $command = NULL;

		$conf =& Config::getInstance();

		if ( $command === NULL ) {
			$command = $conf->get( 'diff_command' );

			if ( $command ) {
				$command = Command::findDiffCommand( $command );
			}
			else if ( ! extension_loaded( 'xdiff' ) && @dl( 'xdiff.so' ) ) {
				$command = 'xdiff.so';
			}
		}
		if ( ! $tag ) {
			$old = strip_tags( $old );
			$new = strip_tags( $new );
	   	}
		if ( $command === 'xdiff.so' ) {
			$diff = xdiff_string_diff( $old, $new );
		}
		else if ( $command ) {
			$tmp_old = tempnam( TEMP_DIR, "PWA_OLD" );
			file_put_contents( $tmp_old, $old );

			$tmp_new = tempnam( TEMP_DIR, "PWA_NEW" );
			file_put_contents( $tmp_new, $new );

			$command = sprintf( "%s %s %s",
				escapeshellcmd( $command ),
				escapeshellarg( $tmp_old ),
				escapeshellarg( $tmp_new )
			);
			$diff = shell_exec( $command );

			is_file( $tmp_old ) && unlink( $tmp_old );
			is_file( $tmp_new ) && unlink( $tmp_new );
		}
		else {
			require_once( LIBS_DIR . 'baselib/diff.inc.php' );
			$diff =	diff_string( $old, $new );
		}
		return trim(
			preg_replace(
				array( "/&nbsp;/", "/^(?:[^>+].*$|[>+]\s*)/m", "/\n+/", "/[\t ]+/" ),
				array( " ",        "",                         "\n",    " "        ),
				$diff
			)
		);
	}

	function findDiffCommand( $diff )
	{
		$diff = escapeshellcmd( preg_replace( '/[\0\s].*$/', '', $diff ) );
		if ( ! is_executable( $diff ) ) {
			$diff = 'diff';
		}
		$result = @shell_exec( $diff . ' --version' );
		if ( ! preg_match( '/^diff/', $result ) ) {
			return FALSE;
		}
		return $diff;
	}

	function diffFile( $old_file, $new_file, $tag = FALSE )
	{
		if ( ! is_readable( $old_file ) || ! is_readable( $new_file ) ) {
			return '';
		}
		return Command::diff(
			file_get_contents( $old_file ),
			file_get_contents( $new_file ),
			$tag
		);
	}

	function existItem( $uri )
	{
		$item =& new Item( $uri );
		if ( ! $uri || ! $item->isFile() ) {
			return FALSE;
		}
		return $uri;
	}

	function getMessage( $id, $option )
	{
		$conf =& Config::getInstance();
		$lang  = $conf->get( 'message_lang' );
		$file  = VIEW_DIR . $lang . '/messages.php';

		$option  = array_pad( $option, 10, '' );
		$message = include( $file );
		if ( isset( $message[$id] ) ) {
			return $message[$id];
		}
		return '';
	}

	function findPhpCli( $php )
	{
		$php = escapeshellcmd( preg_replace( '/[\0\s].*$/', '', $php ) );
		if ( ! is_executable( $php ) ) {
			$php = 'php';
		}
		$result = @shell_exec( $php . ' --version' );
		if ( ! preg_match( '/^PHP ([4-9]\.\d\.\d+((dev|alpha|b(eta)?|RC|pl)\d*)?) *\((cli|cgi(?:-fcgi)?)\)/', $result, $m ) ) {
			return FALSE;
		}
		if ( version_compare( $m[1], '4.3.0', '<' ) ) {
			return FALSE;
		}
		if ( $m[5] !== 'cli' ) {
			$php .= ' -q';
		}
		return $php;
	}

	function redirect( $query = '', $mid = '', $opt = array() )
	{
		$host = get_hostname();
		if ( $mid ) {
			$conf   =& Config::getInstance();
			$message = Command::getMessage( $mid, $opt );
			$id      = uniqid( 'm' );
			$query  .= ( $query ? '&' : '?' ) . 'mid=' . $id;
			$conf->set( 'message', $message );
			$conf->set( 'mid',     $id );
		}
		header( 'Location: ' . $host . WEB_SELF . $query );
		exit;
	}

	function setConfig( $array )
	{
		$conf =& Config::getInstance();
		foreach ( $array as $key => $val ) {
			$conf->set( $key, $val );
		}
		return 'INFO_CHANGE_SETTING';
	}

	function setProxy( $use_proxy, $proxy )
	{
		$conf =& Config::getInstance();
		if ( ! $proxy ) {
			$use_proxy = FALSE;
		}
		else if ( ! Validate::isProxy( $proxy ) ) {
			$proxy     = '';
			$use_proxy = FALSE;
		}
		$conf->set( 'use-proxy', (bool)$use_proxy );
		$conf->set( 'proxy',     $proxy );

		return array( 'INFO_CHANGE_PROXY', $use_proxy, $proxy );
	}

	function updateItem( $uri )
	{
		$result = Command::existItem( $uri );
		if ( ! $result ) {
			return array( 'ERROR_NOT_FILE', $uri );
		}
		$code = Command::checkSite( $uri );
		switch( $code ) {
		case 'OK'           : return array( 'INFO_UPDATE_ITEM',  $uri, $code );
		case 'Not Modified' : return array( 'INFO_NOT_MODIFIED', $uri, $code );
		default             : return array( 'ERROR_UPDATE_ITEM', $uri, $code );
		}
	}

	function upCategory( $id )
	{
		$cate =& Category::getInstance();
		$category = $cate->get( $id );
		$result   = $cate->upCategory( $id );

		return array( $result ? 'INFO_UP_CATEGORY' : 'ERROR_MOVE_CATEGORY', $category );
	}

	function downCategory( $id )
	{
		$cate =& Category::getInstance();
		$category = $cate->get( $id );
		$result   = $cate->downCategory( $id );

		return array( $result ? 'INFO_DOWN_CATEGORY' : 'ERROR_MOVE_CATEGORY', $category );
	}
}
